/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',
    'QDesk.controller.command.scheduler.quartz.jobs.AddJob',
    'QDesk.controller.command.scheduler.quartz.jobs.CloneJob',
    'QDesk.controller.command.scheduler.quartz.jobs.EditJob',
    'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob',
    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.enums.scheduler.quartz.TriggerGroupState',
    'QDesk.enums.scheduler.quartz.TriggerState',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( me.connectionId );

    me._bindEventHandlers( jobsTab );

    jobsTab.setInitialized( true );
  },


  /**
   * Binds events from the jobs tab to their respective handlers.
   */
  _bindEventHandlers : function ( jobsTab )
  {
    var me = this;
    me._bindEventHandlersJobsGrid( jobsTab );

    // individual views
    me._bindEventHandlersHistoryView( jobsTab );
    me._bindEventHandlersTriggersView( jobsTab );
    me._bindEventHandlersStatisticsView( jobsTab );
    me._bindEventHandlersNotificationsView( jobsTab );
  },


  /**
   * Binds the jobs grid events to their handlers.
   */
  _bindEventHandlersJobsGrid : function ( jobsTab )
  {
    var me = this;

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    // refresh button
    jobsGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshJobsGrid( me.connectionId );
        } );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    triggerJobButton.on( 'click', function ()
    {
      var jobDetail = jobsGrid.getSelectionModel().getSelection()[0];
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob', {
        connectionId : me.connectionId,
        jobDetail : jobDetail
      } ).execute();
    }, me );

    var addJobButton = jobsGrid.down( '#addJobButton' );
    addJobButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob', {
        connectionId : me.connectionId
      } ).execute();
    }, me );

    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    deleteJobButton.on( 'click', function ()
    {
      var jobDetail = jobsGrid.getSelectionModel().getSelection()[0];
      me._deleteJob( jobDetail );
    }, me );

    var editJobButton = jobsGrid.down( '#editJobButton' );
    editJobButton.on( 'click', function ()
    {
      var jobDetail = jobsGrid.getSelectionModel().getSelection()[0];
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob', {
        connectionId : me.connectionId,
        jobDetail : jobDetail
      } ).execute();
    }, me );

    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );
    cloneJobButton.on( 'click', function ()
    {
      var jobDetail = jobsGrid.getSelectionModel().getSelection()[0];
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.CloneJob', {
        connectionId : me.connectionId,
        jobDetail : jobDetail
      } ).execute();
    }, me );

    var filterByJobGroupCombo = jobsGrid.down( '#filterByJobGroupCombo' );
    filterByJobGroupCombo.on( 'change', function ( combo, newValue )
    {
      if ( newValue == null )
      {
        resumeJobGroupButton.disable();
        pauseJobGroupButton.disable();
      }
      else
      {
        resumeJobGroupButton.enable();
        pauseJobGroupButton.enable();
      }

      // apply, or clear the filter and load the first page
      jobsGrid.getStore().loadPage( 1, {
        filters : newValue == null ? [] :
            [Ext.create( 'Ext.util.Filter', { property : 'group.name', value : newValue } )]
      } );
    } );

    var resumeJobGroupButton = jobsGrid.down( '#resumeJobGroupButton' );
    resumeJobGroupButton.on( 'click', function ()
    {
      var jobGroupName = filterByJobGroupCombo.getValue();
      me._resumeJobGroup( jobGroupName );
    }, me );

    var pauseJobGroupButton = jobsGrid.down( '#pauseJobGroupButton' );
    pauseJobGroupButton.on( 'click', function ()
    {
      var jobGroupName = filterByJobGroupCombo.getValue();
      me._pauseJobGroup( jobGroupName );
    }, me );

    // event fired by the actioncolumn in the JobsGrid
    jobsGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap )
        {
          me._viewJobDataMap( jobDataMap );
        } );

    jobsGrid.getStore().on( 'beforeload', function ( store, operation )
    {
      jobsGrid.down( '#refreshButton' ).disable();
      filterByJobGroupCombo.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      jobsGrid.getSelectionModel().deselectAll();
    } );

    jobsGrid.getStore().on( 'load', function ( jobDetailStore, records, successful, operation )
    {
      jobsGrid.down( '#refreshButton' ).enable();
      if ( successful )
      {
        addJobButton.enable();
        filterByJobGroupCombo.enable();
      }
      else
      {
        jobDetailStore.removeAll();

        QDesk.util.UxUtils.showErrorDialog(
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.errorLoadingJobsMsg' ) );
      }
    } );

    jobsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
    {
      if ( selectedRecords.length == 0 )
      {
        me._onJobUnselected( jobsTab );
      }
      else
      {
        me._onJobSelected( selectedRecords[0], jobsTab );
      }
    } );
  },


  /**
   * Binds the job execution history panel events to their handlers.
   */
  _bindEventHandlersHistoryView : function ( jobsTab )
  {
    var me = this;

    var historyView = jobsTab.down( '#historyView' );
    historyView.on( 'activate', function ()
    {
      me._refreshHistoryView( me.connectionId, true );
      me._refreshTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshNotificationsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobExecutionHistory &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var execHistoryGrid = jobsTab.down( '#execHistoryGrid' );
      // assert execHistoryGrid != null - the job execution history grid should exist

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch date picker
      execHistoryGrid.down( '#maxFetchDateSelect' ).on( 'changeMaxFetchDate',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch size combo
      execHistoryGrid.down( '#maxFetchSizeSelect' ).on( 'changeMaxFetchSize',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // event fired by the actioncolumn in the JobExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( me.connectionId, exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execHistoryGrid.down( '#refreshButton' ).disable();
        execHistoryGrid.down( '#maxFetchDateSelect' ).disable();
        execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execHistoryGrid.getSelectionModel().deselectAll();
      } );

      execHistoryGrid.getStore().on( 'load', function ( execHistoryStore, records, successful, operation )
      {
        var jobsGrid = jobsTab.down( '#jobsGrid' );
        var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

        if ( jobSelected )
        {
          execHistoryGrid.down( '#refreshButton' ).enable();
          execHistoryGrid.down( '#maxFetchDateSelect' ).enable();
          execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          execHistoryStore.removeAll();

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.errorLoadingExecHistoryMsg' ) );
        }
      } );
    }
  },


  /**
   * Binds the job triggers grid events to their handlers.
   */
  _bindEventHandlersTriggersView : function ( jobsTab )
  {
    var me = this;

    // when the triggers view is activated, we need to refresh it
    var triggersView = jobsTab.down( '#triggersView' );
    triggersView.on( 'activate', function ()
    {
      me._refreshTriggersView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshNotificationsView( me.connectionId, false );
    } );

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );

    var refreshButton = triggersGrid.down( '#refreshButton' );
    refreshButton.on( 'click', function ()
    {
      me._refreshTriggersView( me.connectionId, true );
    }, me );

    var resumeTriggerButton = triggersGrid.down( '#resumeTriggerButton' );
    resumeTriggerButton.on( 'click', function ()
    {
      var me = this;
      var trigger = triggersGrid.getSelectionModel().getSelection()[0];
      me._resumeTrigger( trigger );
    }, me );

    var pauseTriggerButton = triggersGrid.down( '#pauseTriggerButton' );
    pauseTriggerButton.on( 'click', function ()
    {
      var me = this;
      var trigger = triggersGrid.getSelectionModel().getSelection()[0];
      me._pauseTrigger( trigger );
    }, me );

    var deleteTriggerButton = triggersGrid.down( '#deleteTriggerButton' );
    deleteTriggerButton.on( 'click', function ()
    {
      var me = this;
      var trigger = triggersGrid.getSelectionModel().getSelection()[0];
      me._deleteTrigger( trigger );
    }, me );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );
    filterByTriggerGroupCombo.on( 'change', function ( combo, newValue )
    {
      if ( newValue == null )
      {
        resumeTriggerGroupButton.disable();
        pauseTriggerGroupButton.disable();
      }
      else
      {
        // based on the selected trigger group state enable either the resume, or pause button
        var triggerGroup = filterByTriggerGroupCombo.getStore().findRecord( 'name', newValue );
        var triggerGroupState = triggerGroup.get( 'state' );

        switch ( triggerGroupState )
        {
          case QDesk.enums.scheduler.quartz.TriggerGroupState.NORMAL:
            resumeTriggerGroupButton.disable();
            pauseTriggerGroupButton.enable();
            break;

          case QDesk.enums.scheduler.quartz.TriggerGroupState.PAUSED:
            resumeTriggerGroupButton.enable();
            pauseTriggerGroupButton.disable();
            break;

          default:
            Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
        }
      }

      // apply, or clear the filter and load the first page
      triggersGrid.getStore().loadPage( 1, {
        filters : newValue == null ? [] :
            [Ext.create( 'Ext.util.Filter', { property : 'group.name', value : newValue } )]
      } );
    } );

    var resumeTriggerGroupButton = triggersGrid.down( '#resumeTriggerGroupButton' );
    resumeTriggerGroupButton.on( 'click', function ()
    {
      var me = this;
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._resumeTriggerGroup( triggerGroupName );
    }, me );

    var pauseTriggerGroupButton = triggersGrid.down( '#pauseTriggerGroupButton' );
    pauseTriggerGroupButton.on( 'click', function ()
    {
      var me = this;
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._pauseTriggerGroup( triggerGroupName );
    }, me );

    // event fired by the actioncolumn in the JobTriggersGrid
    triggersGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap, effectiveJobDataMap )
        {
          me._viewJobDataMap( jobDataMap, effectiveJobDataMap );
        } );

    triggersGrid.getStore().on( 'beforeload', function ( store, operation )
    {
      refreshButton.disable();
      filterByTriggerGroupCombo.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      triggersGrid.getSelectionModel().deselectAll();
    } );

    triggersGrid.getStore().on( 'load',
        function ( triggerStore, records, successful, operation )
        {
          var jobsGrid = jobsTab.down( '#jobsGrid' );
          var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

          if ( jobSelected )
          {
            refreshButton.enable();
          }

          if ( successful )
          {
            if ( records.length > 0 )
              filterByTriggerGroupCombo.enable();
          }
          else
          {
            triggerStore.removeAll();

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.errorLoadingTriggersMsg' ) );
          }
        } );

    triggersGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length == 0 )
          {
            me._onTriggerUnselected( jobsTab );
          }
          else
          {
            me._onTriggerSelected( selectedRecords[0], jobsTab );
          }
        } );
  },


  /**
   * Binds the job statistics panel events to their handlers.
   */
  _bindEventHandlersStatisticsView : function ( jobsTab )
  {
    var me = this;

    var statisticsView = jobsTab.down( '#statisticsView' );
    statisticsView.on( 'activate', function ()
    {
      me._refreshStatisticsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshTriggersView( me.connectionId, false );
      me._refreshNotificationsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobStatistics &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var statisticsPanel = statisticsView.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );


      // save chart button
      statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/jpeg' );
          } );

      statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/png' );
          } );

      statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/svg+xml' );
          } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );
    }
  },


  /**
   * Binds the job notifications panel events to their handlers.
   */
  _bindEventHandlersNotificationsView : function ( jobsTab )
  {
    var me = this;

    var notificationsView = jobsTab.down( '#notificationsView' );
    notificationsView.on( 'activate', function ()
    {
      me._refreshNotificationsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var notificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

      // refresh button
      notificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          } );

      notificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( jobsTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[0], jobsTab );
        }
      } );

      notificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        notificationRulesGrid.down( '#refreshButton' ).disable();
        notificationRulesGrid.down( '#addNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        notificationRulesGrid.getSelectionModel().deselectAll();
      } );

      notificationRulesGrid.getStore().on( 'load', function ( notificationRuleStore, records, successful, operation )
      {
        var jobsGrid = jobsTab.down( '#jobsGrid' );
        var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

        if ( jobSelected )
        {
          notificationRulesGrid.down( '#refreshButton' ).enable();
          notificationRulesGrid.down( '#addNotificationRuleButton' ).enable();
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          notificationRuleStore.removeAll();

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.errorLoadingNotifRulesMsg' ) );
        }
      } );

      notificationRulesGrid.down( '#addNotificationRuleButton' ).on( 'click', function ()
      {
        var jobsGrid = jobsTab.down( '#jobsGrid' );
        var jobDetail = jobsGrid.getSelectionModel().getSelection()[0];

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionId : me.connectionId,
          jobGroupName : jobDetail.get( 'group' ).name,
          jobName : jobDetail.get( 'name' ),
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#deleteNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRule : notificationRule,
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#editNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#cloneNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      // opens a window with messages enqueued by the notification rule
      notificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages', function ( notificationRuleId )
      {
        Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
          connectionId : me.connectionId,
          creatorId : notificationRuleId,
          creatorType : QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
        } ).execute();
      } );
    }
  },


  /**
   * Refreshes the jobs grid.
   * @param connectionId a connection ID.
   * @private
   */
  _refreshJobsGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job triggers view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshTriggersView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobTriggersView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;

    var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( connectionId );
    var statisticsView = jobsTab.down( '#statisticsView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView', {
      connectionId : connectionId,
      populate : populate,
      maskEl : statisticsView.getEl()
    } ).execute();
  },


  /**
   * Invoked to save the chart in the jobs statistics view.
   * @param connectionId a connection ID.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( connectionId, mimeType )
  {
    var me = this;

    var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( connectionId );
    var chart = jobsTab.down( '#statisticsView #statisticsPanel #chartContainer chart' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked when a job in the JobsGrid in the JobsTab is selected.
   * @param jobDetail a QDesk.model.quartz.JobDetail instance representing the selected job.
   * @param jobsTab the jobs tab.
   */
  _onJobSelected : function ( jobDetail, jobsTab )
  {
    var me = this;

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    //var addJobButton = jobsGrid.down( '#addJobButton' );
    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    var editJobButton = jobsGrid.down( '#editJobButton' );
    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );

    triggerJobButton.enable();
    //addJobButton.enable();
    deleteJobButton.enable();
    editJobButton.enable();
    cloneJobButton.enable();

    var views = jobsTab.down( '#views' );
    // assert views != null - the views should exist

    var connectionId = jobsTab.getConnectionId();
    var activeView = views.getActiveTab();

    me._refreshHistoryView( connectionId, activeView.itemId == 'historyView' );
    me._refreshTriggersView( connectionId, activeView.itemId == 'triggersView' );
    me._refreshStatisticsView( connectionId, activeView.itemId == 'statisticsView' );
    me._refreshNotificationsView( connectionId, activeView.itemId == 'notificationsView' );
  },


  /**
   * Invoked when a job in the JobsGrid in the JobsTab is unselected.
   * @param jobsTab a JobsTab.
   */
  _onJobUnselected : function ( jobsTab )
  {
    var me = this;

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    var addJobButton = jobsGrid.down( '#addJobButton' );
    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    var editJobButton = jobsGrid.down( '#editJobButton' );
    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );

    triggerJobButton.disable();
    addJobButton.disable();
    editJobButton.disable();
    deleteJobButton.disable();
    cloneJobButton.disable();

    var connectionId = jobsTab.getConnectionId();

    me._refreshHistoryView( connectionId, false );
    me._refreshTriggersView( connectionId, false );
    me._refreshStatisticsView( connectionId, false );
    me._refreshNotificationsView( connectionId, false );
  },


  /**
   * Invoked when a trigger in the JobTriggersGrid in the JobsTab is selected.
   * @param trigger a QDesk.model.quartz.Trigger instance representing the selected trigger.
   * @param jobsTab a JobsTab.
   */
  _onTriggerSelected : function ( trigger, jobsTab )
  {
    var me = this;

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );
    var triggerState = trigger.get( 'state' );

    switch ( triggerState )
    {
      case QDesk.enums.scheduler.quartz.TriggerState.NONE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.NORMAL:
        triggersGrid.down( '#resumeTriggerButton' ).disable();
        triggersGrid.down( '#pauseTriggerButton' ).enable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.PAUSED:
        triggersGrid.down( '#resumeTriggerButton' ).enable();
        triggersGrid.down( '#pauseTriggerButton' ).disable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.COMPLETE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.ERROR:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.BLOCKED:
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger state value: ' + triggerState );
    }

    triggersGrid.down( '#deleteTriggerButton' ).enable();
  },


  /**
   * Invoked when a trigger in the JobTriggersGrid in the JobsTab is unselected.
   * @param jobsTab a JobsTab.
   */
  _onTriggerUnselected : function ( jobsTab )
  {
    var me = this;

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );

    triggersGrid.down( '#resumeTriggerButton' ).disable();
    triggersGrid.down( '#pauseTriggerButton' ).disable();
    triggersGrid.down( '#deleteTriggerButton' ).disable();
  },


  /**
   * Invoked when a notification rule record in the JobNotificationRulesGrid in the JobNotificationsView is selected.
   * @param notificationRule a QDesk.model.quartz.ExecNotificationRule instance representing the selected
   * notification rule.
   * @param jobsTab the JobsTab.
   */
  _onNotificationRuleSelected : function ( notificationRule, jobsTab )
  {
    var me = this;

    var notificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the JobNotificationRulesGrid in the JobNotificationsView is unselected.
   * @param jobsTab the JobsTab.
   */
  _onNotificationRuleUnselected : function ( jobsTab )
  {
    var me = this;

    var notificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
  },


  _deleteJob : function ( jobDetail )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob', {
      connectionId : me.connectionId,
      jobDetail : jobDetail,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _resumeJobGroup : function ( jobGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeJobGroup', {
      connectionId : me.connectionId,
      jobGroupName : jobGroupName,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseJobGroup : function ( jobGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseJobGroup', {
      connectionId : me.connectionId,
      jobGroupName : jobGroupName,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( me.connectionId, true );
      }
    } ).execute();
  },


  _resumeTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( me.connectionId, true );
      }
    } ).execute();
  },


  _deleteTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.DeleteTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( me.connectionId, true );
      }
    } ).execute();
  },


  _resumeTriggerGroup : function ( triggerGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( me.connectionId, true );
      }
    } ).execute();
  },


  _pauseTriggerGroup : function ( triggerGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( me.connectionId, true );
      }
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   * @param effectiveJobDataMap an effective job data map.
   */
  _viewJobDataMap : function ( jobDataMap, effectiveJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      jobDataMap : jobDataMap,
      effectiveJobDataMap : effectiveJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param connectionId a connection ID.
   * @param exception the exception data.
   */
  _viewException : function ( connectionId, exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryUserData', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
